@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2019 by the deal.II authors
+// Copyright (C) 2009 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -29,20 +29,68 @@ DEAL_II_NAMESPACE_OPEN
 /**
  * Definition of a finite element space with zero degrees of freedom and that,
  * consequently, can only represent a single function: the zero function.
- * This class is
- * useful (in the context of an hp method) to represent empty cells in the
- * triangulation on which no degrees of freedom should be allocated, or to
- * describe a field that is extended by zero to a part of the domain where we
- * don't need it.  Thus a triangulation may be divided into two regions: an
- * active region where normal elements are used, and an inactive region where
- * FE_Nothing elements are used.  The hp::DoFHandler will therefore assign no
- * degrees of freedom to the FE_Nothing cells, and this subregion is therefore
- * implicitly deleted from the computation. step-10 and step-46 show use cases
- * for this element. An interesting application for this element is also
- * presented in the paper A. Cangiani, J. Chapman, E. Georgoulis, M. Jensen:
- * <b>Implementation of the Continuous-Discontinuous Galerkin Finite Element
- * Method</b>, arXiv:1201.2878v1 [math.NA], 2012 (see
- * http://arxiv.org/abs/1201.2878).
+ *
+ * This class is useful (in the context of an hp-method) to represent empty
+ * cells in the triangulation on which no degrees of freedom should be
+ * allocated, or to describe a field that is extended by zero to a part of the
+ * domain where we don't need it. Thus a triangulation may be divided into two
+ * regions: an active region where normal elements are used, and an inactive
+ * region where FE_Nothing elements are used. The DoFHandler will therefore
+ * assign no degrees of freedom to the FE_Nothing cells, and this subregion is
+ * therefore implicitly deleted from the computation. step-10 and step-46 show
+ * use cases for this element. An interesting application for this element is
+ * also presented in the paper @cite Cangiani2012.
+ *
+ *
+ * <h3>FE_Nothing as seen as a function space</h3>
+ *
+ * Finite elements are often best interpreted as forming a
+ * [function space](https://en.wikipedia.org/wiki/Function_space), i.e., a
+ * set of functions that form a
+ * [vector space](https://en.wikipedia.org/wiki/Vector_space). One can indeed
+ * interpret FE_Nothing in this light: It corresponds to the function space
+ * $V_h=\{0\}$, i.e., the set of functions that are zero everywhere.
+ * (The constructor can take an argument that, if greater than one, extends
+ * the space to one of vector-valued functions with more than one component,
+ * with all components equal to zero everywhere.) Indeed, this is a vector
+ * space since every linear combination of elements in the vector space is
+ * also an element in the vector space, as is every multiple of the single
+ * element zero. It is obvious that the function space has no degrees of
+ * freedom, thus the name of the class.
+ *
+ *
+ * <h3>FE_Nothing in combination with other elements</h3>
+ *
+ * In situations such as those of step-46, one uses FE_Nothing on cells
+ * where one is not interested in a solution variable. For example, in fluid
+ * structure interaction problems, the fluid velocity is only defined on
+ * cells inside the fluid part of the domain. One then uses FE_Nothing
+ * on cells in the solid part of the domain to describe the finite element
+ * space for the velocity. In other words, the velocity lives everywhere
+ * conceptually, but it is identically zero in those parts of the domain
+ * where it is not of interest and doesn't use up any degrees of freedom
+ * there.
+ *
+ * The question is what happens at the interface between areas where one
+ * is interested in the solution (and uses a "normal" finite element) and
+ * where one is not interested (and uses FE_Nothing): Should the solution
+ * at that interface be zero -- i.e., we consider a "continuous" finite
+ * element field that happens to be zero in that area where FE_Nothing
+ * is used -- or is there no requirement for continuity at the interface.
+ * In the deal.II language, this is encoded by what the function
+ * FiniteElement::compare_for_domination() returns: If the FE_Nothing
+ * "dominates", then the solution must be zero at the interface; if it
+ * does not, then there is no requirement and one can think of FE_Nothing
+ * as a function space that is in general discontinuous (i.e., there is
+ * no requirement for any kind of continuity at cell interfaces) but on
+ * every cell equal to zero.
+ *
+ * A constructor argument denotes whether the element should be considered
+ * dominating or not. The default is for it not to dominate, i.e.,
+ * FE_Nothing is treated as a discontinuous element.
+ *
+ *
+ * <h3>FE_Nothing in the context of hanging nodes</h3>
  *
  * Note that some care must be taken that the resulting mesh topology
  * continues to make sense when FE_Nothing elements are introduced. This is
@@ -76,23 +124,33 @@ DEAL_II_NAMESPACE_OPEN
  * @endcode
  * The distinction lies in the mixed nature of the child faces, a case we have
  * not implemented as of yet.
- *
- * @author Joshua White, Wolfgang Bangerth
  */
 template <int dim, int spacedim = dim>
 class FE_Nothing : public FiniteElement<dim, spacedim>
 {
 public:
   /**
-   * Constructor. First argument denotes the number of components to give this
-   * finite element (default = 1).
+   * Constructor.
    *
-   * Second argument decides whether FE_Nothing will dominate any other FE in
-   * compare_for_domination() (default = false). Therefore at interfaces where,
-   * for example, a Q1 meets an FE_Nothing, we will force the traces of the two
-   * functions to be the same. Because the FE_Nothing encodes a space that is
-   * zero everywhere, this means that the Q1 field will be forced to become zero
-   * at this interface.
+   * @param[in] type Specifies the reference-cell type.
+   *
+   * @param[in] n_components Denotes the number of
+   * vector components to give this finite element. The default is one.
+   *
+   * @param[in] dominate Decides whether FE_Nothing will dominate
+   * any other FE in compare_for_domination() (with the default being `false`).
+   * Therefore at interfaces where, for example, a $Q_1$ meets an FE_Nothing, we
+   * will force the traces of the two functions to be the same. Because the
+   * FE_Nothing encodes a space that is zero everywhere, this means that the
+   * $Q_1$ field will be forced to become zero at this interface. See also the
+   * discussion in the general documentation of this class.
+   */
+  FE_Nothing(const ReferenceCell &type,
+             const unsigned int   n_components = 1,
+             const bool           dominate     = false);
+
+  /**
+   * Same as above but for a hypercube reference-cell type.
    */
   FE_Nothing(const unsigned int n_components = 1, const bool dominate = false);
 
@@ -100,8 +158,17 @@ public:
   clone() const override;
 
   /**
-   * Return a string that uniquely identifies a finite element. In this case
-   * it is <code>FE_Nothing@<dim@></code>.
+   * Return a string that uniquely identifies a finite element. The name is
+   * <tt>FE_Nothing@<dim,spacedim@>(type, n_components, dominating)</tt> where
+   * <tt>dim</tt>, <tt>spacedim</tt>, <tt>type</tt>, and <tt>n_components</tt>
+   * are all specified by the constructor or type signature with the following
+   * exceptions:
+   * <ol>
+   *   <li>If <tt>spacedim == dim</tt> then that field is not printed.</li>
+   *   <li>If <tt>type</tt> is a hypercube then that field is not printed.</li>
+   *   <li>If <tt>n_components == 1</tt> then that field is not printed.</li>
+   *   <li>If <tt>dominate == false</tt> then that field is not printed.</li>
+   * </ol>
    */
   virtual std::string
   get_name() const override;
@@ -135,11 +202,13 @@ public:
                                                                        spacedim>
       &output_data) const override;
 
+  using FiniteElement<dim, spacedim>::fill_fe_face_values;
+
   virtual void
   fill_fe_face_values(
     const typename Triangulation<dim, spacedim>::cell_iterator &cell,
     const unsigned int                                          face_no,
-    const Quadrature<dim - 1> &                                 quadrature,
+    const hp::QCollection<dim - 1> &                            quadrature,
     const Mapping<dim, spacedim> &                              mapping,
     const typename Mapping<dim, spacedim>::InternalDataBase &mapping_internal,
     const dealii::internal::FEValuesImplementation::MappingRelatedData<dim,
@@ -192,6 +261,8 @@ public:
    * argument in the constructor @p dominate is true. When this argument is
    * false and @p fe_other is also of type FE_Nothing(), either element can
    * dominate. Otherwise there are no_requirements.
+   *
+   * See also the discussion in the general documentation of this class.
    */
   virtual FiniteElementDomination::Domination
   compare_for_domination(const FiniteElement<dim, spacedim> &fe_other,
@@ -208,8 +279,8 @@ public:
     const FiniteElement<dim, spacedim> &fe_other) const override;
 
   virtual std::vector<std::pair<unsigned int, unsigned int>>
-  hp_quad_dof_identities(
-    const FiniteElement<dim, spacedim> &fe_other) const override;
+  hp_quad_dof_identities(const FiniteElement<dim, spacedim> &fe_other,
+                         const unsigned int face_no = 0) const override;
 
   virtual bool
   hp_constraints_are_implemented() const override;
@@ -234,9 +305,9 @@ public:
    */
 
   virtual void
-  get_face_interpolation_matrix(
-    const FiniteElement<dim, spacedim> &source_fe,
-    FullMatrix<double> &                interpolation_matrix) const override;
+  get_face_interpolation_matrix(const FiniteElement<dim, spacedim> &source_fe,
+                                FullMatrix<double> &interpolation_matrix,
+                                const unsigned int  face_no = 0) const override;
 
 
   /**
@@ -252,7 +323,8 @@ public:
   get_subface_interpolation_matrix(
     const FiniteElement<dim, spacedim> &source_fe,
     const unsigned int                  index,
-    FullMatrix<double> &                interpolation_matrix) const override;
+    FullMatrix<double> &                interpolation_matrix,
+    const unsigned int                  face_no = 0) const override;
 
   /**
    * @return true if the FE dominates any other.
@@ -260,16 +332,6 @@ public:
   bool
   is_dominating() const;
 
-  /**
-   * Comparison operator. In addition to the fields already checked by
-   * FiniteElement::operator==(), this operator also checks for equality
-   * of the arguments passed to the constructors of the current object
-   * as well as the object against which the comparison is done (which
-   * for this purpose obviously also needs to be of type FE_Nothing).
-   */
-  virtual bool
-  operator==(const FiniteElement<dim, spacedim> &fe) const override;
-
 private:
   /**
    * If true, this element will dominate any other apart from itself in
