@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2020 by the deal.II authors
+// Copyright (C) 2004 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,6 +19,8 @@
 
 #  include <deal.II/base/config.h>
 
+#  include <deal.II/base/subscriptor.h>
+
 #  ifdef DEAL_II_WITH_PETSC
 
 #    include <deal.II/lac/exceptions.h>
@@ -52,20 +54,19 @@ namespace PETScWrappers
    * work for parallel jobs, such as for example the ILU preconditioner.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
    */
-  class PreconditionerBase
+  class PreconditionBase : public Subscriptor
   {
   public:
     /**
      * Constructor.
      */
-    PreconditionerBase();
+    PreconditionBase();
 
     /**
      * Destructor.
      */
-    virtual ~PreconditionerBase();
+    virtual ~PreconditionBase();
 
     /**
      * Destroys the preconditioner, leaving an object like just after having
@@ -130,13 +131,12 @@ namespace PETScWrappers
    * preconditioner.
    *
    * See the comment in the base class
-   * @ref PreconditionerBase
+   * @ref PreconditionBase
    * for when this preconditioner may or may not work.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
    */
-  class PreconditionJacobi : public PreconditionerBase
+  class PreconditionJacobi : public PreconditionBase
   {
   public:
     /**
@@ -166,7 +166,7 @@ namespace PETScWrappers
      * Intended to be used with SLEPc objects.
      */
     PreconditionJacobi(
-      const MPI_Comm        communicator,
+      const MPI_Comm &      communicator,
       const AdditionalData &additional_data = AdditionalData());
 
     /**
@@ -214,13 +214,12 @@ namespace PETScWrappers
    * the relevant section of the PETSc manual, but is not implemented here.
    *
    * See the comment in the base class
-   * @ref PreconditionerBase
+   * @ref PreconditionBase
    * for when this preconditioner may or may not work.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
    */
-  class PreconditionBlockJacobi : public PreconditionerBase
+  class PreconditionBlockJacobi : public PreconditionBase
   {
   public:
     /**
@@ -249,7 +248,7 @@ namespace PETScWrappers
      * Intended to be used with SLEPc objects.
      */
     PreconditionBlockJacobi(
-      const MPI_Comm        communicator,
+      const MPI_Comm &      communicator,
       const AdditionalData &additional_data = AdditionalData());
 
 
@@ -287,9 +286,8 @@ namespace PETScWrappers
    * @note Only works in serial with a PETScWrappers::SparseMatrix.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
    */
-  class PreconditionSOR : public PreconditionerBase
+  class PreconditionSOR : public PreconditionBase
   {
   public:
     /**
@@ -348,9 +346,8 @@ namespace PETScWrappers
    * @note Only works in serial with a PETScWrappers::SparseMatrix.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
    */
-  class PreconditionSSOR : public PreconditionerBase
+  class PreconditionSSOR : public PreconditionBase
   {
   public:
     /**
@@ -408,13 +405,12 @@ namespace PETScWrappers
    * each processor.
    *
    * See the comment in the base class
-   * @ref PreconditionerBase
+   * @ref PreconditionBase
    * for when this preconditioner may or may not work.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
    */
-  class PreconditionEisenstat : public PreconditionerBase
+  class PreconditionEisenstat : public PreconditionBase
   {
   public:
     /**
@@ -474,9 +470,8 @@ namespace PETScWrappers
    * @note Only works in serial with a PETScWrappers::SparseMatrix.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
    */
-  class PreconditionICC : public PreconditionerBase
+  class PreconditionICC : public PreconditionBase
   {
   public:
     /**
@@ -535,9 +530,8 @@ namespace PETScWrappers
    * @note Only works in serial with a PETScWrappers::SparseMatrix.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, Timo Heister, 2004, 2011
    */
-  class PreconditionILU : public PreconditionerBase
+  class PreconditionILU : public PreconditionBase
   {
   public:
     /**
@@ -595,15 +589,14 @@ namespace PETScWrappers
    * (depending on the settings) performs an exact factorization of the
    * matrix, so it is not necessary to wrap it in an iterative solver. This
    * class is typically used with SolverPreOnly to get a direct
-   * solver. Alternatively, you can use PreconditionerBase::vmult() directly.
+   * solver. Alternatively, you can use PreconditionBase::vmult() directly.
    *
    * @note This is not a parallel preconditioner so it only works in serial
    * computations with a single processor.
    *
    * @ingroup PETScWrappers
-   * @author Oliver Kayser-Herold, 2004
    */
-  class PreconditionLU : public PreconditionerBase
+  class PreconditionLU : public PreconditionBase
   {
   public:
     /**
@@ -681,9 +674,8 @@ namespace PETScWrappers
    * step-40 for an example.
    *
    * @ingroup PETScWrappers
-   * @author Timo Heister, 2010
    */
-  class PreconditionBoomerAMG : public PreconditionerBase
+  class PreconditionBoomerAMG : public PreconditionBase
   {
   public:
     /**
@@ -692,21 +684,51 @@ namespace PETScWrappers
      */
     struct AdditionalData
     {
+      /**
+       * Defines the available relaxation types for BoomerAMG.
+       */
+      enum class RelaxationType
+      {
+        Jacobi,
+        sequentialGaussSeidel,
+        seqboundaryGaussSeidel,
+        SORJacobi,
+        backwardSORJacobi,
+        symmetricSORJacobi,
+        l1scaledSORJacobi,
+        GaussianElimination,
+        l1GaussSeidel,
+        backwardl1GaussSeidel,
+        CG,
+        Chebyshev,
+        FCFJacobi,
+        l1scaledJacobi,
+        None
+      };
+
       /**
        * Constructor. Note that BoomerAMG offers a lot more options to set
        * than what is exposed here.
        */
-      AdditionalData(const bool         symmetric_operator = false,
-                     const double       strong_threshold   = 0.25,
-                     const double       max_row_sum        = 0.9,
-                     const unsigned int aggressive_coarsening_num_levels = 0,
-                     const bool         output_details = false);
+      AdditionalData(
+        const bool           symmetric_operator               = false,
+        const double         strong_threshold                 = 0.25,
+        const double         max_row_sum                      = 0.9,
+        const unsigned int   aggressive_coarsening_num_levels = 0,
+        const bool           output_details                   = false,
+        const RelaxationType relaxation_type_up   = RelaxationType::SORJacobi,
+        const RelaxationType relaxation_type_down = RelaxationType::SORJacobi,
+        const RelaxationType relaxation_type_coarse =
+          RelaxationType::GaussianElimination,
+        const unsigned int n_sweeps_coarse = 1,
+        const double       tol             = 0.0,
+        const unsigned int max_iter        = 1,
+        const bool         w_cycle         = false);
 
       /**
        * Set this flag to true if you have a symmetric system matrix and you
        * want to use a solver which assumes a symmetric preconditioner like
-       * CG. The relaxation is done with SSOR/Jacobi when set to true and with
-       * SOR/Jacobi otherwise.
+       * CG.
        */
       bool symmetric_operator;
 
@@ -742,6 +764,42 @@ namespace PETScWrappers
        * preconditioner is constructed.
        */
       bool output_details;
+
+      /**
+       * Choose relaxation type up.
+       */
+      RelaxationType relaxation_type_up;
+
+      /**
+       * Choose relaxation type down.
+       */
+      RelaxationType relaxation_type_down;
+
+      /**
+       * Choose relaxation type coarse.
+       */
+      RelaxationType relaxation_type_coarse;
+
+      /**
+       * Choose number of sweeps on coarse grid.
+       */
+      unsigned int n_sweeps_coarse;
+
+      /**
+       * Choose BommerAMG tolerance.
+       */
+      double tol;
+
+      /**
+       * Choose BommerAMG maximum number of cycles.
+       */
+      unsigned int max_iter;
+
+      /**
+       * Defines whether a w-cycle should be used instead of the standard
+       * setting of a v-cycle.
+       */
+      bool w_cycle;
     };
 
     /**
@@ -763,7 +821,7 @@ namespace PETScWrappers
      * Intended to be used with SLEPc objects.
      */
     PreconditionBoomerAMG(
-      const MPI_Comm        communicator,
+      const MPI_Comm &      communicator,
       const AdditionalData &additional_data = AdditionalData());
 
 
@@ -812,9 +870,8 @@ namespace PETScWrappers
    * The preconditioner does support parallel distributed computations.
    *
    * @ingroup PETScWrappers
-   * @author Martin Steigemann, 2012
    */
-  class PreconditionParaSails : public PreconditionerBase
+  class PreconditionParaSails : public PreconditionBase
   {
   public:
     /**
@@ -923,9 +980,8 @@ namespace PETScWrappers
    * A class that implements a non-preconditioned method.
    *
    * @ingroup PETScWrappers
-   * @author Martin Steigemann, 2012
    */
-  class PreconditionNone : public PreconditionerBase
+  class PreconditionNone : public PreconditionBase
   {
   public:
     /**
@@ -966,6 +1022,12 @@ namespace PETScWrappers
      */
     AdditionalData additional_data;
   };
+
+  /**
+   * Alias for backwards-compatibility.
+   * @deprecated Use PETScWrappers::PreconditionBase instead.
+   */
+  using PreconditionerBase DEAL_II_DEPRECATED_EARLY = PreconditionBase;
 } // namespace PETScWrappers
 
 
