@@ -36,6 +36,13 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+#ifdef signals
+#  error \
+    "The name 'signals' is already defined. You are most likely using the QT library \
+and using the 'signals' keyword. You can either #include the Qt headers (or any conflicting headers) \
+*after* the deal.II headers or you can define the 'QT_NO_KEYWORDS' macro and use the 'Q_SIGNALS' macro."
+#endif
+
 /*!@addtogroup mg */
 /*@{*/
 
@@ -70,6 +77,10 @@ namespace mg
      * This signal is triggered before (@p before is true) and after (@p before
      * is
      * false) the call to the coarse solver on @p level.
+     *
+     * The coarse solve will be done with ``defect[leve]`` and returned in
+     * ``solution[level]``, which can be inspected by the user using this
+     * signal.
      */
     boost::signals2::signal<void(const bool before, const unsigned int level)>
       coarse_solve;
@@ -78,6 +89,9 @@ namespace mg
      * This signal is triggered before (@p before is true) and after (@p before
      * is false) the call to MGTransfer::restrict_and_add() which restricts a
      * vector from @p level to the next coarser one (@p level - 1).
+     *
+     * The vector ``defect[level-1]`` will be updated between these two
+     * triggers and can be inspected by the user using this signal.
      */
     boost::signals2::signal<void(const bool before, const unsigned int level)>
       restriction;
@@ -94,6 +108,9 @@ namespace mg
      * This signal is triggered before (@p before is true) and after (@p before
      * is false) the call to a pre-smoothing step via MGPreSmoother::apply() on
      * @p level.
+     *
+     * The smoother result will be stored in ``solution[level]`` and can be
+     * inspected by the user using this signal.
      */
     boost::signals2::signal<void(const bool before, const unsigned int level)>
       pre_smoother_step;
@@ -130,8 +147,6 @@ namespace mg
  * will have to set up quite a few auxiliary objects before you can use it.
  * Unfortunately, it seems that this can be avoided only be restricting the
  * flexibility of this class in an unacceptable way.
- *
- * @author Guido Kanschat, 1999 - 2005
  */
 template <typename VectorType>
 class Multigrid : public Subscriptor
@@ -447,8 +462,6 @@ private:
  * If VectorType is in fact a block vector and the TRANSFER object supports
  * use of a separate DoFHandler for each block, this class also allows
  * to be initialized with a separate DoFHandler for each block.
- *
- * @author Guido Kanschat, Daniel Arndt, 1999, 2000, 2001, 2002, 2017
  */
 template <int dim, typename VectorType, class TRANSFER>
 class PreconditionMG : public Subscriptor
@@ -533,7 +546,7 @@ public:
   /**
    * Return the MPI communicator object in use with this preconditioner.
    */
-  const MPI_Comm &
+  MPI_Comm
   get_mpi_communicator() const;
 
   /**
@@ -850,7 +863,7 @@ PreconditionMG<dim, VectorType, TRANSFER>::locally_owned_domain_indices(
 
 
 template <int dim, typename VectorType, class TRANSFER>
-const MPI_Comm &
+MPI_Comm
 PreconditionMG<dim, VectorType, TRANSFER>::get_mpi_communicator() const
 {
   // currently parallel GMG works with parallel triangulations only,
