@@ -41,7 +41,6 @@ namespace Physics
      *
      * @relatesalso Tensor
      * @relatesalso SymmetricTensor
-     * @author Jean-Paul Pelteret, Andrew McBride, 2016
      */
     template <int dim>
     class StandardTensors
@@ -70,7 +69,7 @@ namespace Physics
        * matrix, then this simply corresponds to the identity matrix.
        */
       static DEAL_II_CONSTEXPR const SymmetricTensor<2, dim> I
-#ifdef DEAL_II_HAVE_CXX14_CONSTEXPR
+#ifndef DEAL_II_CXX14_CONSTEXPR_BUG
         = unit_symmetric_tensor<dim>()
 #endif
         ;
@@ -103,7 +102,7 @@ namespace Physics
        * operator.
        */
       static DEAL_II_CONSTEXPR const SymmetricTensor<4, dim> S
-#ifdef DEAL_II_HAVE_CXX14_CONSTEXPR
+#ifndef DEAL_II_CXX14_CONSTEXPR_BUG
         = identity_tensor<dim>()
 #endif
         ;
@@ -119,7 +118,7 @@ namespace Physics
        * @f]
        */
       static DEAL_II_CONSTEXPR const SymmetricTensor<4, dim> IxI
-#ifdef DEAL_II_HAVE_CXX14_CONSTEXPR
+#ifndef DEAL_II_CXX14_CONSTEXPR_BUG
         = outer_product(unit_symmetric_tensor<dim>(),
                         unit_symmetric_tensor<dim>())
 #endif
@@ -153,12 +152,12 @@ namespace Physics
        *  \dealcoloneq \{ \bullet \} - \frac{1}{\textrm{dim}}
        *  \left[ \{ \bullet \} : \mathbf{I} \right]\mathbf{I}
        *  = \mathcal{P}^{T} : \{ \bullet \}
-       *  = \texttt{dev\_P} \left( \{ \bullet \} \right)
+       *  = \mathtt{dev\_P} \left( \{ \bullet \} \right)
        * @f]
        * and, therefore,
        * @f[
-       * \texttt{dev\_P} \left( \{ \bullet \} \right) : \mathbf{I}
-       *   = \textrm{trace}(\texttt{dev\_P} \left( \{ \bullet \} \right)) = 0 \,
+       * \mathtt{dev\_P} \left( \{ \bullet \} \right) : \mathbf{I}
+       *   = \mathrm{trace}(\mathtt{dev\_P} \left( \{ \bullet \} \right)) = 0 \,
        * .
        * @f]
        *
@@ -169,7 +168,7 @@ namespace Physics
        * @dealiiHolzapfelA{232,6.105}
        */
       static DEAL_II_CONSTEXPR const SymmetricTensor<4, dim> dev_P
-#ifdef DEAL_II_HAVE_CXX14_CONSTEXPR
+#ifndef DEAL_II_CXX14_CONSTEXPR_BUG
         = deviator_tensor<dim>()
 #endif
         ;
@@ -201,12 +200,12 @@ namespace Physics
        *  \{ \bullet \} : \hat{\mathcal{P}}
        *    \dealcoloneq J^{-2/\textrm{dim}} \left[ \{ \bullet \} -
        * \frac{1}{\textrm{dim}}\left[\mathbf{C} : \{ \bullet \}\right]
-       * \mathbf{C}^{-1} \right] = \texttt{Dev\_P} \left( \{ \bullet \} \right)
+       * \mathbf{C}^{-1} \right] = \mathtt{Dev\_P} \left( \{ \bullet \} \right)
        * \, .
        * @f]
        * It can therefore be readily shown that
        * @f[
-       *  \texttt{Dev\_P} \left( \{ \bullet \} \right) : \mathbf{C} = 0 \, .
+       *  \mathtt{Dev\_P} \left( \{ \bullet \} \right) : \mathbf{C} = 0 \, .
        * @f]
        *
        * @note It may be observed that we have defined the tensor as the
@@ -244,7 +243,7 @@ namespace Physics
        *  \hat{\mathcal{P}}^{T} : \{ \bullet \}
        *    = J^{-2/\textrm{dim}} \left[ \{ \bullet \} - \frac{1}{\textrm{dim}}
        * \left[\mathbf{C}^{-1} : \{ \bullet \}\right] \mathbf{C} \right] =
-       * \texttt{Dev\_P\_T} \{ \bullet \}
+       * \mathtt{Dev\_P\_T} \{ \bullet \}
        * @f]
        */
       template <typename Number>
@@ -374,7 +373,8 @@ DEAL_II_CONSTEXPR SymmetricTensor<2, dim, Number>
                   Physics::Elasticity::StandardTensors<dim>::ddet_F_dC(
   const Tensor<2, dim, Number> &F)
 {
-  return Number(0.5) * determinant(F) * symmetrize(invert(transpose(F) * F));
+  return internal::NumberType<Number>::value(0.5 * determinant(F)) *
+         symmetrize(invert(transpose(F) * F));
 }
 
 
