@@ -1,6 +1,6 @@
 //-----------------------------------------------------------
 //
-//    Copyright (C) 2017 - 2020 by the deal.II authors
+//    Copyright (C) 2017 - 2021 by the deal.II authors
 //
 //    This file is part of the deal.II library.
 //
@@ -17,8 +17,10 @@
 #define dealii_sundials_kinsol_h
 
 #include <deal.II/base/config.h>
+
 #ifdef DEAL_II_WITH_SUNDIALS
 
+
 #  include <deal.II/base/conditional_ostream.h>
 #  include <deal.II/base/exceptions.h>
 #  include <deal.II/base/logstream.h>
@@ -31,7 +33,9 @@
 #  include <boost/signals2.hpp>
 
 #  include <kinsol/kinsol.h>
-#  include <kinsol/kinsol_impl.h>
+#  if DEAL_II_SUNDIALS_VERSION_LT(4, 1, 0)
+#    include <kinsol/kinsol_impl.h>
+#  endif
 #  include <nvector/nvector_serial.h>
 #  include <sundials/sundials_math.h>
 #  include <sundials/sundials_types.h>
@@ -47,20 +51,22 @@ DEAL_II_NAMESPACE_OPEN
 namespace SUNDIALS
 {
   /**
-   * Interface to SUNDIALS non linear solver (KINSOL).
+   * Interface to SUNDIALS' nonlinear solver (KINSOL).
    *
    * KINSOL is a solver for nonlinear algebraic systems in residual form $F(u)
-   * = 0$ or fixed point form $G(u) = u$. It includes a Newton-Krylov solver
+   * = 0$ or fixed point form $G(u) = u$, where $u$ is a vector which we will
+   * assume to be in ${\mathbb R}^n$ or ${\mathbb C}^n$, but that may also have
+   * a block structure and may be distributed in parallel computations; the
+   * functions $F$ and $G$ satisfy $F,G:{\mathbb R}^N \to{\mathbb R}^N$ or
+   * $F,G:{\mathbb C}^N \to{\mathbb C}^N$. It includes a Newton-Krylov solver
    * as well as Picard and fixed point solvers, both of which can be
    * accelerated with Anderson acceleration. KINSOL is based on the previous
-   * Fortran package NKSOL of Brown and Saad.
+   * Fortran package NKSOL of Brown and Saad. An example of using KINSOL
+   * can be found in the step-77 tutorial program.
    *
    * KINSOL's Newton solver employs the inexact Newton method. As this solver
    * is intended mainly for large systems, the user is required to provide
-   * their own solver function. If a solver function is not provided, the
-   * internal dense solver of KINSOL is used. Be warned that this solver
-   * computes the Jacobian approximately, and may be efficient only for small
-   * systems.
+   * their own solver function.
    *
    * At the highest level, KINSOL implements the following iteration
    * scheme:
@@ -127,22 +133,6 @@ namespace SUNDIALS
    * algorithm is that the full Newton step tends to be taken close to the
    * solution.
    *
-   * As a user option, KINSOL permits the application of inequality
-   * constraints, $u_i > 0$ and $u_i < 0$, as well as $u_i \geq 0$ and $u_i
-   * \leq 0$, where $u_i$ is the $i$-th component of $u$. Any such constraint,
-   * or no constraint, may be imposed on each component by providing the
-   * optional functions
-   * - get_lower_than_zero_constrained_entries()
-   * - get_greater_than_zero_constrained_entries()
-   * - get_lower_equal_than_zero_constrained_entries()
-   * - get_greater_or_equal_than_zero_constrained_entries()
-   *
-   * KINSOL will reduce step lengths in order to ensure that no constraint is
-   * violated. Specifically, if a new Newton iterate will violate a constraint,
-   * the maximum step length along the Newton direction that will satisfy all
-   * constraints is found, and $\delta_n$ is scaled to take a step of that
-   * length.
-   *
    * The basic fixed-point iteration scheme implemented in KINSOL is given by:
    * - Set $u_0 =$ an initial guess
    * - For $n = 0, 1, 2, \dots$ until convergence do:
@@ -171,28 +161,23 @@ namespace SUNDIALS
    * or
    *  - iteration_function;
    *
-   * Specifying residual() allows the user to use Newton strategies (i.e.,
-   * $F(u)=0$ will be solved), while specifying iteration_function(), fixed
-   * point iteration or Picard iteration will be used (i.e., $G(u)=u$ will be
-   * solved).
+   * Specifying residual() allows the user to use Newton and Picard strategies
+   * (i.e., $F(u)=0$ will be solved), while specifying iteration_function(), a
+   * fixed point iteration will be used (i.e., $G(u)=u$ will be solved).
    *
-   * If the use of a Newton method is desired, then the user should also supply
-   *  - solve_jacobian_system;
+   * If the use of a Newton or Picard method is desired, then the user should
+   * also supply
+   *  - solve_jacobian_system or solve_with_jacobian;
    * and optionally
    *  - setup_jacobian;
    *
-   * If the solve_jacobian_system() function is not supplied, then KINSOL will
-   * use its internal dense solver for Newton methods, with approximate
-   * Jacobian. This may be very expensive for large systems. Fixed point
-   * iteration does not require the solution of any linear system.
+   * Fixed point iteration does not require the solution of any linear system.
    *
    * Also the following functions could be rewritten, to provide additional
    * scaling factors for both the solution and the residual evaluation during
    * convergence checks:
    *  - get_solution_scaling;
    *  - get_function_scaling;
-   *
-   * @author Luca Heltai, 2017.
    */
   template <typename VectorType = Vector<double>>
   class KINSOL
@@ -236,7 +221,7 @@ namespace SUNDIALS
        * @param strategy Solution strategy
        * @param maximum_non_linear_iterations Maximum number of nonlinear
        * iterations
-       * @param function_tolerance Function norm stopping tolerance
+       * @param function_tolerance %Function norm stopping tolerance
        * @param step_tolerance Scaled step stopping tolerance
        *
        * Newton parameters:
@@ -248,7 +233,7 @@ namespace SUNDIALS
        * @param dq_relative_error Relative error for different quotient
        * computation
        *
-       * Linesearch parameters:
+       * Line search parameters:
        *
        * @param maximum_beta_failures Maximum number of beta-condition failures
        *
@@ -256,29 +241,16 @@ namespace SUNDIALS
        *
        * @param anderson_subspace_size Anderson acceleration subspace size
        */
-      AdditionalData(
-        // Global parameters
-        const SolutionStrategy &strategy                      = linesearch,
-        const unsigned int      maximum_non_linear_iterations = 200,
-        const double            function_tolerance            = 0.0,
-        const double            step_tolerance                = 0.0,
-        const bool              no_init_setup                 = false,
-        const unsigned int      maximum_setup_calls           = 0,
-        const double            maximum_newton_step           = 0.0,
-        const double            dq_relative_error             = 0.0,
-        const unsigned int      maximum_beta_failures         = 0,
-        const unsigned int      anderson_subspace_size        = 0)
-        : strategy(strategy)
-        , maximum_non_linear_iterations(maximum_non_linear_iterations)
-        , function_tolerance(function_tolerance)
-        , step_tolerance(step_tolerance)
-        , no_init_setup(no_init_setup)
-        , maximum_setup_calls(maximum_setup_calls)
-        , maximum_newton_step(maximum_newton_step)
-        , dq_relative_error(dq_relative_error)
-        , maximum_beta_failures(maximum_beta_failures)
-        , anderson_subspace_size(anderson_subspace_size)
-      {}
+      AdditionalData(const SolutionStrategy &strategy = linesearch,
+                     const unsigned int maximum_non_linear_iterations = 200,
+                     const double       function_tolerance            = 0.0,
+                     const double       step_tolerance                = 0.0,
+                     const bool         no_init_setup                 = false,
+                     const unsigned int maximum_setup_calls           = 0,
+                     const double       maximum_newton_step           = 0.0,
+                     const double       dq_relative_error             = 0.0,
+                     const unsigned int maximum_beta_failures         = 0,
+                     const unsigned int anderson_subspace_size        = 0);
 
       /**
        * Add all AdditionalData() parameters to the given ParameterHandler
@@ -314,58 +286,12 @@ namespace SUNDIALS
        * a parameter file using `prm`. The values of the parameter will be
        * updated whenever the content of `prm` is updated.
        *
-       * Make sure that this class lives longer than `prm`. Undefined behaviour
+       * Make sure that this class lives longer than `prm`. Undefined behavior
        * will occur if you destroy this class, and then parse a parameter file
        * using `prm`.
        */
       void
-      add_parameters(ParameterHandler &prm)
-      {
-        static std::string strategy_str("newton");
-        prm.add_parameter("Solution strategy",
-                          strategy_str,
-                          "Choose among newton|linesearch|fixed_point|picard",
-                          Patterns::Selection(
-                            "newton|linesearch|fixed_point|picard"));
-        prm.add_action("Solution strategy", [&](const std::string &value) {
-          if (value == "newton")
-            strategy = newton;
-          else if (value == "linesearch")
-            strategy = linesearch;
-          else if (value == "fixed_point")
-            strategy = fixed_point;
-          else if (value == "picard")
-            strategy = picard;
-          else
-            Assert(false, ExcInternalError());
-        });
-        prm.add_parameter("Maximum number of nonlinear iterations",
-                          maximum_non_linear_iterations);
-        prm.add_parameter("Function norm stopping tolerance",
-                          function_tolerance);
-        prm.add_parameter("Scaled step stopping tolerance", step_tolerance);
-
-        prm.enter_subsection("Newton parameters");
-        prm.add_parameter("No initial matrix setup", no_init_setup);
-        prm.add_parameter("Maximum iterations without matrix setup",
-                          maximum_setup_calls);
-        prm.add_parameter("Maximum allowable scaled length of the Newton step",
-                          maximum_newton_step);
-        prm.add_parameter("Relative error for different quotient computation",
-                          dq_relative_error);
-        prm.leave_subsection();
-
-        prm.enter_subsection("Linesearch parameters");
-        prm.add_parameter("Maximum number of beta-condition failures",
-                          maximum_beta_failures);
-        prm.leave_subsection();
-
-
-        prm.enter_subsection("Fixed point and Picard parameters");
-        prm.add_parameter("Anderson acceleration subspace size",
-                          anderson_subspace_size);
-        prm.leave_subsection();
-      }
+      add_parameters(ParameterHandler &prm);
 
       /**
        * The solution strategy to use. If you choose SolutionStrategy::newton
@@ -411,7 +337,9 @@ namespace SUNDIALS
        * The maximum number of nonlinear iterations that can be
        * performed between calls to the setup_jacobian() function.
        *
-       * If set to zero, default values provided by KINSOL will be used.
+       * If set to zero, default values provided by KINSOL will be used,
+       * and in practice this often means that KINSOL will re-use a
+       * Jacobian matrix computed in one iteration for later iterations.
        */
       unsigned int maximum_setup_calls;
 
@@ -456,7 +384,7 @@ namespace SUNDIALS
      * @param mpi_comm MPI communicator
      */
     KINSOL(const AdditionalData &data     = AdditionalData(),
-           const MPI_Comm        mpi_comm = MPI_COMM_WORLD);
+           const MPI_Comm &      mpi_comm = MPI_COMM_WORLD);
 
     /**
      * Destructor.
@@ -473,14 +401,18 @@ namespace SUNDIALS
 
     /**
      * A function object that users need to supply and that is intended to
-     * reinit the given vector.
+     * reinitize the given vector to its correct size, block structure (if
+     * block vectors are used), and MPI communicator (if the vector is
+     * distributed across multiple processors using MPI), along with any
+     * other properties necessary.
      */
     std::function<void(VectorType &)> reinit_vector;
 
     /**
      * A function object that users should supply and that is intended to
-     * compute the residual dst = F(src). This function is only used if the
-     * SolutionStrategy::newton or SolutionStrategy::linesearch are specified.
+     * compute the residual `dst = F(src)`. This function is only used if the
+     * SolutionStrategy::newton or SolutionStrategy::linesearch strategies
+     * were selected.
      *
      * This function should return:
      * - 0: Success
@@ -493,21 +425,21 @@ namespace SUNDIALS
 
     /**
      * A function object that users should supply and that is intended to
-     * compute the iteration function G(u) for the fixed point and Picard
+     * compute the iteration function $G(u)$ for the fixed point and Picard
      * iteration. This function is only used if the
-     * SolutionStrategy::fixed_point or SolutionStrategy::picard are specified.
+     * SolutionStrategy::fixed_point or SolutionStrategy::picard strategies
+     * were selected.
      *
      * This function should return:
      * - 0: Success
      * - >0: Recoverable error (KINSOL will try to change its internal
      * parameters and attempt a new solution step)
-     * - <0: Unrecoverable error the computation will be aborted and an
+     * - <0: Unrecoverable error; the computation will be aborted and an
      * assertion will be thrown.
      */
     std::function<int(const VectorType &src, VectorType &dst)>
       iteration_function;
 
-
     /**
      * A function object that users may supply and that is intended to
      * prepare the linear solver for subsequent calls to
@@ -519,27 +451,32 @@ namespace SUNDIALS
      * SolutionStrategy that has been selected.
      *
      * In the cases strategy = SolutionStrategy::newton or
-     * SolutionStrategy::linesearch, A is the Jacobian $J = \partial F/\partial
-     * u$. If strategy = SolutionStrategy::picard, A is the approximate
-     * Jacobian matrix $L$. If strategy = SolutionStrategy::fixed_point, then
-     * linear systems do not arise, and this function is never called.
+     * SolutionStrategy::linesearch, $A$ is the Jacobian $J = \partial
+     * F/\partial u$. If strategy = SolutionStrategy::picard, $A$ is the
+     * approximate Jacobian matrix $L$. If strategy =
+     * SolutionStrategy::fixed_point, then linear systems do not arise, and this
+     * function is never called.
      *
      * The setup_jacobian() function may call a user-supplied function, or a
      * function within the linear solver module, to compute Jacobian-related
      * data that is required by the linear solver. It may also preprocess that
      * data as needed for solve_jacobian_system(), which may involve calling a
-     * generic function (such as for LU factorization). This data may be
-     * intended either for direct use (in a direct linear solver) or for use in
-     * a preconditioner (in a preconditioned iterative linear solver).
+     * generic function (such as for LU factorization) or, more generally,
+     * build preconditioners from the assembled Jacobian. In any case, the
+     * data so generated may then be used whenever a linear system is solved.
      *
-     * The setup_jacobian() function is not called at every Newton iteration,
+     * The point of this function is that
+     * setup_jacobian() function is not called at every Newton iteration,
      * but only as frequently as the solver determines that it is appropriate
      * to perform the setup task. In this way, Jacobian-related data generated
      * by setup_jacobian() is expected to be used over a number of Newton
-     * iterations.
+     * iterations. KINSOL determines itself when it is beneficial to regenerate
+     * the Jacobian and associated information (such as preconditioners
+     * computed for the Jacobian), thereby saving the effort to regenerate
+     * the Jacobian matrix and a preconditioner for it whenever possible.
      *
-     * @param current_u Current value of u
-     * @param current_f Current value of F(u) or G(u)
+     * @param current_u Current value of $u$
+     * @param current_f Current value of $F(u)$ or $G(u)$
      *
      * This function should return:
      * - 0: Success
@@ -552,34 +489,100 @@ namespace SUNDIALS
       setup_jacobian;
 
     /**
+     * @deprecated Versions of SUNDIALS after 4.0 no longer provide all
+     *   of the information necessary for this callback (see below). Use the
+     *   `solve_with_jacobian` callback described below.
+     *
      * A function object that users may supply and that is intended to solve
-     * the Jacobian linear system. This function will be called by KINSOL
-     * (possibly several times) after setup_jacobian() has been called at least
-     * once. KINSOL tries to do its best to call setup_jacobian() the minimum
-     * amount of times. If convergence can be achieved without updating the
-     * Jacobian, then KINSOL does not call setup_jacobian() again. If, on the
-     * contrary, internal KINSOL convergence tests fail, then KINSOL calls
-     * again setup_jacobian() with updated vectors and coefficients so that
+     * a linear system with the Jacobian matrix. This function will be called by
+     * KINSOL (possibly several times) after setup_jacobian() has been called at
+     * least once. KINSOL tries to do its best to call setup_jacobian() the
+     * minimum number of times. If convergence can be achieved without updating
+     * the Jacobian, then KINSOL does not call setup_jacobian() again. If, on
+     * the contrary, internal KINSOL convergence tests fail, then KINSOL calls
+     * setup_jacobian() again with updated vectors and coefficients so that
      * successive calls to solve_jacobian_systems() lead to better convergence
      * in the Newton process.
      *
-     * If you do not specify a solve_jacobian_system() function, then only a
+     * If you do not specify a `solve_jacobian_system` or `solve_with_jacobian`
+     * function, then only a fixed point iteration strategy can be used. Notice
+     * that this may not converge, or may converge very slowly.
+     *
+     * A call to this function should store in `dst` the result of $J^{-1}$
+     * applied to `rhs`, i.e., `J*dst = rhs`. It is the user's responsibility
+     * to set up proper solvers and preconditioners inside this function
+     * (or in the `setup_jacobian` callback above).
+     *
+     *
+     * Arguments to the function are:
+     *
+     * @param[in] ycur The current $y$ vector for the current KINSOL
+     * internal step. In the documentation above, this $y$ vector is generally
+     * denoted by $u$.
+     * @param[in] fcur The current value of the implicit right-hand side at
+     * `ycur`, $f_I (t_n, ypred)$.
+     * @param[in] rhs The system right hand side to solve for
+     * @param[out] dst The solution of $J^{-1} * src$
+     *
+     * This function should return:
+     * - 0: Success
+     * - >0: Recoverable error (KINSOL will try to change its internal
+     * parameters and attempt a new solution step)
+     * - <0: Unrecoverable error the computation will be aborted and an
+     * assertion will be thrown.
+     *
+     * @warning Starting with SUNDIALS 4.1, SUNDIALS no longer provides the
+     *   `ycur` and `fcur` variables -- only `rhs` is provided and `dst`
+     *   needs to be returned. The first two arguments will therefore be
+     *   empty vectors in that case. In practice, that means that one
+     *   can no longer compute a Jacobian matrix for the current iterate
+     *   within this function. Rather, this has to happen inside the
+     *   `setup_jacobian` function above that receives this information.
+     *   If it is important that the Jacobian corresponds to the *current*
+     *   iterate (rather than a re-used Jacobian matrix that had been
+     *   computed in a previous iteration and that therefore corresponds
+     *   to a *previous* iterate), then you will also have to set the
+     *   AdditionalData::maximum_newton_step variable to one, indicating
+     *   that the Jacobian should be re-computed in every iteration.
+     */
+    DEAL_II_DEPRECATED_EARLY
+    std::function<int(const VectorType &ycur,
+                      const VectorType &fcur,
+                      const VectorType &rhs,
+                      VectorType &      dst)>
+      solve_jacobian_system;
+
+    /**
+     * A function object that users may supply and that is intended to solve
+     * a linear system with the Jacobian matrix. This function will be called by
+     * KINSOL (possibly several times) after setup_jacobian() has been called at
+     * least once. KINSOL tries to do its best to call setup_jacobian() the
+     * minimum number of times. If convergence can be achieved without updating
+     * the Jacobian, then KINSOL does not call setup_jacobian() again. If, on
+     * the contrary, internal KINSOL convergence tests fail, then KINSOL calls
+     * setup_jacobian() again with updated vectors and coefficients so that
+     * successive calls to solve_jacobian_systems() lead to better convergence
+     * in the Newton process.
+     *
+     * If you do not specify a `solve_with_jacobian` function, then only a
      * fixed point iteration strategy can be used. Notice that this may not
      * converge, or may converge very slowly.
      *
      * A call to this function should store in `dst` the result of $J^{-1}$
-     * applied to `src`, i.e., `J*dst = src`. It is the users responsibility
-     * to set up proper solvers and preconditioners inside this function.
+     * applied to `rhs`, i.e., `J*dst = rhs`. It is the user's responsibility
+     * to set up proper solvers and preconditioners inside this function
+     * (or in the `setup_jacobian` callback above). The function attached
+     * to this callback is also provided with a tolerance to the linear solver,
+     * indicating that it is not necessary to solve the linear system with
+     * the Jacobian matrix exactly, but only to a tolerance that KINSOL will
+     * adapt over time.
      *
+     * Arguments to the function are:
      *
-     * Arguments to the function are
-     *
-     * @param[in] ycur  is the current $y$ vector for the current KINSOL
-     * internal step
-     * @param[in] fcur  is the current value of the implicit right-hand side at
-     * ycur, $f_I (t_n, ypred)$.
-     * @param[in] rhs  the system right hand side to solve for
-     * @param[out] dst the solution of $A^{-1} * src$
+     * @param[in] rhs The system right hand side to solve for.
+     * @param[out] dst The solution of $J^{-1} * src$.
+     * @param[in] tolerance The tolerance with which to solve the linear system
+     *   of equations.
      *
      * This function should return:
      * - 0: Success
@@ -588,17 +591,47 @@ namespace SUNDIALS
      * - <0: Unrecoverable error the computation will be aborted and an
      * assertion will be thrown.
      */
-    std::function<int(const VectorType &ycur,
-                      const VectorType &fcur,
-                      const VectorType &rhs,
-                      VectorType &      dst)>
-      solve_jacobian_system;
+    std::function<
+      int(const VectorType &rhs, VectorType &dst, const double tolerance)>
+      solve_with_jacobian;
 
     /**
      * A function object that users may supply and that is intended to return a
      * vector whose components are the weights used by KINSOL to compute the
      * vector norm of the solution. The implementation of this function is
      * optional, and it is used only if implemented.
+     *
+     * The intent for this scaling factor is for problems in which the different
+     * components of a solution have vastly different numerical magnitudes --
+     * typically because they have different physical units and represent
+     * different things. For example, if one were to solve a nonlinear Stokes
+     * problem, the solution vector has components that correspond to velocities
+     * and other components that correspond to pressures. These have different
+     * physical units and depending on which units one chooses, they may have
+     * roughly comparable numerical sizes or maybe they don't. To give just one
+     * example, in simulations of flow in the Earth's interior, one has
+     * velocities on the order of maybe ten centimeters per year, and pressures
+     * up to around 100 GPa. If one expresses this in SI units, this corresponds
+     * to velocities of around $0.000,000,003=3 \times 10^{-9}$ m/s, and
+     * pressures around $10^9 \text{kg}/\text{m}/\text{s}^2$, i.e., vastly
+     * different. In such cases, computing the $l_2$ norm of a solution-type
+     * vector (e.g., the difference between the previous and the current
+     * solution) makes no sense because the norm will either be dominated by the
+     * velocity components or the pressure components. The scaling vector this
+     * function returns is intended to provide each component of the solution
+     * with a scaling factor that is generally chosen as as the inverse of a
+     * "typical velocity" or "typical pressure" so that upon multiplication of a
+     * vector component by the corresponding scaling vector component, one
+     * obtains a number that is of order of magnitude of one (i.e., a reasonably
+     * small multiple of one times the typical velocity/pressure). The KINSOL
+     * manual states this as follows: "The user should supply values $D_u$,
+     * which are diagonal elements of the scaling matrix such that $D_u U$ has
+     * all components roughly the same magnitude when $U$ is close to a
+     * solution".
+     *
+     * If no function is provided to a KINSOL object, then this is interpreted
+     * as implicitly saying that all of these scaling factors should be
+     * considered as one.
      */
     std::function<VectorType &()> get_solution_scaling;
 
@@ -608,6 +641,13 @@ namespace SUNDIALS
      * vector norm of the function evaluation away from the solution. The
      * implementation of this function is optional, and it is used only if
      * implemented.
+     *
+     * The point of this function and the scaling vector it returns is similar
+     * to the one discussed above for `get_solution_scaling`, except that it is
+     * for a vector that scales the components of the function $F(U)$, rather
+     * than the components of $U$, when computing norms. As above, if no
+     * function is provided, then this is equivalent to using a scaling vector
+     * whose components are all equal to one.
      */
     std::function<VectorType &()> get_function_scaling;
 
@@ -617,7 +657,7 @@ namespace SUNDIALS
     DeclException1(ExcKINSOLError,
                    int,
                    << "One of the SUNDIALS KINSOL internal functions "
-                   << " returned a negative error code: " << arg1
+                   << "returned a negative error code: " << arg1
                    << ". Please consult SUNDIALS manual.");
 
 
@@ -649,26 +689,6 @@ namespace SUNDIALS
      */
     void *kinsol_mem;
 
-    /**
-     * KINSOL solution vector.
-     */
-    N_Vector solution;
-
-    /**
-     * KINSOL solution scale.
-     */
-    N_Vector u_scale;
-
-    /**
-     * KINSOL f scale.
-     */
-    N_Vector f_scale;
-
-    /**
-     * MPI communicator. SUNDIALS solver runs happily in parallel.
-     */
-    MPI_Comm communicator;
-
     /**
      * Memory pool of vectors.
      */
@@ -679,7 +699,7 @@ namespace SUNDIALS
 
 
 DEAL_II_NAMESPACE_CLOSE
+
 #endif
-
 
 #endif
