@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2018 by the deal.II authors
+// Copyright (C) 2009 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,18 +14,20 @@
 // ---------------------------------------------------------------------
 
 
-#include <deal.II/base/std_cxx14/memory.h>
-
 #include <deal.II/fe/fe_nothing.h>
 
+#include <memory>
+
 DEAL_II_NAMESPACE_OPEN
 
 
 template <int dim, int spacedim>
-FE_Nothing<dim, spacedim>::FE_Nothing(const unsigned int n_components,
-                                      const bool         dominate)
+FE_Nothing<dim, spacedim>::FE_Nothing(const ReferenceCell &type,
+                                      const unsigned int   n_components,
+                                      const bool           dominate)
   : FiniteElement<dim, spacedim>(
       FiniteElementData<dim>(std::vector<unsigned>(dim + 1, 0),
+                             type,
                              n_components,
                              0,
                              FiniteElementData<dim>::unknown),
@@ -33,6 +35,10 @@ FE_Nothing<dim, spacedim>::FE_Nothing(const unsigned int n_components,
       std::vector<ComponentMask>())
   , dominate(dominate)
 {
+  Assert(n_components >= 1,
+         ExcMessage("A finite element needs to have at least one "
+                    "vector component."));
+
   // in most other elements we have to set up all sorts of stuff
   // here. there isn't much that we have to do here; in particular,
   // we can simply leave the restriction and prolongation matrices
@@ -41,11 +47,22 @@ FE_Nothing<dim, spacedim>::FE_Nothing(const unsigned int n_components,
 }
 
 
+
+template <int dim, int spacedim>
+FE_Nothing<dim, spacedim>::FE_Nothing(const unsigned int n_components,
+                                      const bool         dominate)
+  : FE_Nothing<dim, spacedim>(ReferenceCells::get_hypercube<dim>(),
+                              n_components,
+                              dominate)
+{}
+
+
+
 template <int dim, int spacedim>
 std::unique_ptr<FiniteElement<dim, spacedim>>
 FE_Nothing<dim, spacedim>::clone() const
 {
-  return std_cxx14::make_unique<FE_Nothing<dim, spacedim>>(*this);
+  return std::make_unique<FE_Nothing<dim, spacedim>>(*this);
 }
 
 
@@ -55,16 +72,24 @@ std::string
 FE_Nothing<dim, spacedim>::get_name() const
 {
   std::ostringstream namebuf;
-  namebuf << "FE_Nothing<" << dim << ">(";
+  namebuf << "FE_Nothing<" << Utilities::dim_string(dim, spacedim) << ">(";
+
+  std::vector<std::string> name_components;
+  if (this->reference_cell() != ReferenceCells::get_hypercube<dim>())
+    name_components.push_back(this->reference_cell().to_string());
   if (this->n_components() > 1)
+    name_components.push_back(std::to_string(this->n_components()));
+  if (dominate)
+    name_components.emplace_back("dominating");
+
+  for (const std::string &comp : name_components)
     {
-      namebuf << this->n_components();
-      if (dominate)
-        namebuf << ", dominating";
+      namebuf << comp;
+      if (comp != name_components.back())
+        namebuf << ", ";
     }
-  else if (dominate)
-    namebuf << "dominating";
   namebuf << ")";
+
   return namebuf.str();
 }
 
@@ -103,7 +128,7 @@ FE_Nothing<dim, spacedim>::get_data(
   // Create a default data object.  Normally we would then
   // need to resize things to hold the appropriate numbers
   // of dofs, but in this case all data fields are empty.
-  return std_cxx14::make_unique<
+  return std::make_unique<
     typename FiniteElement<dim, spacedim>::InternalDataBase>();
 }
 
@@ -135,7 +160,7 @@ void
 FE_Nothing<dim, spacedim>::fill_fe_face_values(
   const typename Triangulation<dim, spacedim>::cell_iterator &,
   const unsigned int,
-  const Quadrature<dim - 1> &,
+  const hp::QCollection<dim - 1> &,
   const Mapping<dim, spacedim> &,
   const typename Mapping<dim, spacedim>::InternalDataBase &,
   const dealii::internal::FEValuesImplementation::MappingRelatedData<dim,
@@ -182,28 +207,6 @@ FE_Nothing<dim, spacedim>::is_dominating() const
 
 
 
-template <int dim, int spacedim>
-bool
-FE_Nothing<dim, spacedim>::
-operator==(const FiniteElement<dim, spacedim> &f) const
-{
-  // Compare fields stored in the base class
-  if (!(this->FiniteElement<dim, spacedim>::operator==(f)))
-    return false;
-
-  // Then make sure the other object is really of type FE_Nothing,
-  // and compare the data that has been passed to both objects'
-  // constructors.
-  if (const FE_Nothing<dim, spacedim> *f_nothing =
-        dynamic_cast<const FE_Nothing<dim, spacedim> *>(&f))
-    return ((dominate == f_nothing->dominate) &&
-            (this->components == f_nothing->components));
-  else
-    return false;
-}
-
-
-
 template <int dim, int spacedim>
 FiniteElementDomination::Domination
 FE_Nothing<dim, spacedim>::compare_for_domination(
@@ -220,11 +223,12 @@ FE_Nothing<dim, spacedim>::compare_for_domination(
     // if it does and the other is FE_Nothing, either can dominate
     return FiniteElementDomination::either_element_can_dominate;
   else
-    // otherwise we dominate whatever fe is provided
+    // otherwise we dominate whatever FE is provided
     return FiniteElementDomination::this_element_dominates;
 }
 
 
+
 template <int dim, int spacedim>
 std::vector<std::pair<unsigned int, unsigned int>>
 FE_Nothing<dim, spacedim>::hp_vertex_dof_identities(
@@ -254,7 +258,8 @@ FE_Nothing<dim, spacedim>::hp_line_dof_identities(
 template <int dim, int spacedim>
 std::vector<std::pair<unsigned int, unsigned int>>
 FE_Nothing<dim, spacedim>::hp_quad_dof_identities(
-  const FiniteElement<dim, spacedim> & /*fe_other*/) const
+  const FiniteElement<dim, spacedim> & /*fe_other*/,
+  const unsigned int) const
 {
   // the FE_Nothing has no
   // degrees of freedom, so there
@@ -295,7 +300,8 @@ template <int dim, int spacedim>
 void
 FE_Nothing<dim, spacedim>::get_face_interpolation_matrix(
   const FiniteElement<dim, spacedim> & /*source_fe*/,
-  FullMatrix<double> &interpolation_matrix) const
+  FullMatrix<double> &interpolation_matrix,
+  const unsigned int) const
 {
   // since this element has no face dofs, the
   // interpolation matrix is necessarily empty
@@ -313,7 +319,8 @@ void
 FE_Nothing<dim, spacedim>::get_subface_interpolation_matrix(
   const FiniteElement<dim, spacedim> & /*source_fe*/,
   const unsigned int /*index*/,
-  FullMatrix<double> &interpolation_matrix) const
+  FullMatrix<double> &interpolation_matrix,
+  const unsigned int) const
 {
   // since this element has no face dofs, the
   // interpolation matrix is necessarily empty
