@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2020 by the deal.II authors
+// Copyright (C) 2016 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,14 +15,12 @@
 
 
 
+#include <deal.II/base/std_cxx17/cmath.h>
 #include <deal.II/base/thread_management.h>
 
 #include <deal.II/fe/fe_series.h>
 
 #include <iostream>
-#ifdef DEAL_II_WITH_GSL
-#  include <gsl/gsl_sf_legendre.h>
-#endif
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -42,26 +40,15 @@ namespace
   double
   Lh(const Point<dim> &x_q, const TableIndices<dim> &indices)
   {
-#ifdef DEAL_II_WITH_GSL
     double res = 1.0;
     for (unsigned int d = 0; d < dim; d++)
       {
         const double x = 2.0 * (x_q[d] - 0.5);
         Assert((x_q[d] <= 1.0) && (x_q[d] >= 0.), ExcLegendre(d, x_q[d]));
-        const int ind = indices[d];
-        res *= std::sqrt(2.0) * gsl_sf_legendre_Pl(ind, x);
+        const unsigned int ind = indices[d];
+        res *= std::sqrt(2.0) * std_cxx17::legendre(ind, x);
       }
     return res;
-
-#else
-
-    (void)x_q;
-    (void)indices;
-    AssertThrow(false,
-                ExcMessage("deal.II has to be configured with GSL "
-                           "in order to use Legendre transformation."));
-    return 0;
-#endif
   }
 
 
@@ -87,14 +74,16 @@ namespace
   integrate(const FiniteElement<dim, spacedim> &fe,
             const Quadrature<dim> &             quadrature,
             const TableIndices<dim> &           indices,
-            const unsigned int                  dof)
+            const unsigned int                  dof,
+            const unsigned int                  component)
   {
     double sum = 0;
     for (unsigned int q = 0; q < quadrature.size(); ++q)
       {
         const Point<dim> &x_q = quadrature.point(q);
-        sum +=
-          Lh(x_q, indices) * fe.shape_value(dof, x_q) * quadrature.weight(q);
+        sum += Lh(x_q, indices) *
+               fe.shape_value_component(dof, x_q, component) *
+               quadrature.weight(q);
       }
     return sum * multiplier(indices);
   }
@@ -112,19 +101,25 @@ namespace
     const hp::FECollection<1, spacedim> &fe_collection,
     const hp::QCollection<1> &           q_collection,
     const unsigned int                   fe,
+    const unsigned int                   component,
     std::vector<FullMatrix<double>> &    legendre_transform_matrices)
   {
     AssertIndexRange(fe, fe_collection.size());
 
     if (legendre_transform_matrices[fe].m() == 0)
       {
-        legendre_transform_matrices[fe].reinit(n_coefficients_per_direction[fe],
-                                               fe_collection[fe].dofs_per_cell);
+        legendre_transform_matrices[fe].reinit(
+          n_coefficients_per_direction[fe],
+          fe_collection[fe].n_dofs_per_cell());
 
         for (unsigned int k = 0; k < n_coefficients_per_direction[fe]; ++k)
-          for (unsigned int j = 0; j < fe_collection[fe].dofs_per_cell; ++j)
-            legendre_transform_matrices[fe](k, j) = integrate(
-              fe_collection[fe], q_collection[fe], TableIndices<1>(k), j);
+          for (unsigned int j = 0; j < fe_collection[fe].n_dofs_per_cell(); ++j)
+            legendre_transform_matrices[fe](k, j) =
+              integrate(fe_collection[fe],
+                        q_collection[fe],
+                        TableIndices<1>(k),
+                        j,
+                        component);
       }
   }
 
@@ -135,6 +130,7 @@ namespace
     const hp::FECollection<2, spacedim> &fe_collection,
     const hp::QCollection<2> &           q_collection,
     const unsigned int                   fe,
+    const unsigned int                   component,
     std::vector<FullMatrix<double>> &    legendre_transform_matrices)
   {
     AssertIndexRange(fe, fe_collection.size());
@@ -143,18 +139,20 @@ namespace
       {
         legendre_transform_matrices[fe].reinit(
           Utilities::fixed_power<2>(n_coefficients_per_direction[fe]),
-          fe_collection[fe].dofs_per_cell);
+          fe_collection[fe].n_dofs_per_cell());
 
         unsigned int k = 0;
         for (unsigned int k1 = 0; k1 < n_coefficients_per_direction[fe]; ++k1)
           for (unsigned int k2 = 0; k2 < n_coefficients_per_direction[fe];
                ++k2, k++)
-            for (unsigned int j = 0; j < fe_collection[fe].dofs_per_cell; ++j)
+            for (unsigned int j = 0; j < fe_collection[fe].n_dofs_per_cell();
+                 ++j)
               legendre_transform_matrices[fe](k, j) =
                 integrate(fe_collection[fe],
                           q_collection[fe],
                           TableIndices<2>(k1, k2),
-                          j);
+                          j,
+                          component);
       }
   }
 
@@ -165,6 +163,7 @@ namespace
     const hp::FECollection<3, spacedim> &fe_collection,
     const hp::QCollection<3> &           q_collection,
     const unsigned int                   fe,
+    const unsigned int                   component,
     std::vector<FullMatrix<double>> &    legendre_transform_matrices)
   {
     AssertIndexRange(fe, fe_collection.size());
@@ -173,19 +172,21 @@ namespace
       {
         legendre_transform_matrices[fe].reinit(
           Utilities::fixed_power<3>(n_coefficients_per_direction[fe]),
-          fe_collection[fe].dofs_per_cell);
+          fe_collection[fe].n_dofs_per_cell());
 
         unsigned int k = 0;
         for (unsigned int k1 = 0; k1 < n_coefficients_per_direction[fe]; ++k1)
           for (unsigned int k2 = 0; k2 < n_coefficients_per_direction[fe]; ++k2)
             for (unsigned int k3 = 0; k3 < n_coefficients_per_direction[fe];
                  ++k3, k++)
-              for (unsigned int j = 0; j < fe_collection[fe].dofs_per_cell; ++j)
+              for (unsigned int j = 0; j < fe_collection[fe].n_dofs_per_cell();
+                   ++j)
                 legendre_transform_matrices[fe](k, j) =
                   integrate(fe_collection[fe],
                             q_collection[fe],
                             TableIndices<3>(k1, k2, k3),
-                            j);
+                            j,
+                            component);
       }
   }
 } // namespace
@@ -198,16 +199,28 @@ namespace FESeries
   Legendre<dim, spacedim>::Legendre(
     const std::vector<unsigned int> &      n_coefficients_per_direction,
     const hp::FECollection<dim, spacedim> &fe_collection,
-    const hp::QCollection<dim> &           q_collection)
+    const hp::QCollection<dim> &           q_collection,
+    const unsigned int                     component_)
     : n_coefficients_per_direction(n_coefficients_per_direction)
     , fe_collection(&fe_collection)
     , q_collection(q_collection)
     , legendre_transform_matrices(fe_collection.size())
+    , component(component_ != numbers::invalid_unsigned_int ? component_ : 0)
   {
     Assert(n_coefficients_per_direction.size() == fe_collection.size() &&
              n_coefficients_per_direction.size() == q_collection.size(),
            ExcMessage("All parameters are supposed to have the same size."));
 
+    if (fe_collection[0].n_components() > 1)
+      Assert(
+        component_ != numbers::invalid_unsigned_int,
+        ExcMessage(
+          "For vector-valued problems, you need to explicitly specify for "
+          "which vector component you will want to do a Fourier decomposition "
+          "by setting the 'component' argument of this constructor."));
+
+    AssertIndexRange(component, fe_collection[0].n_components());
+
     // reserve sufficient memory
     const unsigned int max_n_coefficients_per_direction =
       *std::max_element(n_coefficients_per_direction.cbegin(),
@@ -241,7 +254,8 @@ namespace FESeries
       (n_coefficients_per_direction == legendre.n_coefficients_per_direction) &&
       (*fe_collection == *(legendre.fe_collection)) &&
       (q_collection == legendre.q_collection) &&
-      (legendre_transform_matrices == legendre.legendre_transform_matrices));
+      (legendre_transform_matrices == legendre.legendre_transform_matrices) &&
+      (component == legendre.component));
   }
 
 
@@ -257,6 +271,7 @@ namespace FESeries
                          *fe_collection,
                          q_collection,
                          fe,
+                         component,
                          legendre_transform_matrices);
       });
 
@@ -291,6 +306,7 @@ namespace FESeries
                      *fe_collection,
                      q_collection,
                      cell_active_fe_index,
+                     component,
                      legendre_transform_matrices);
 
     const FullMatrix<CoefficientType> &matrix =
