@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2020 by the deal.II authors
+// Copyright (C) 2003 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -165,7 +165,7 @@ namespace
       const MGConstrainedDoFs,
       MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number>>>
                                                 mg_constrained_dofs,
-    const MPI_Comm                              mpi_communicator,
+    const MPI_Comm &                            mpi_communicator,
     const bool                                  transfer_solution_vectors,
     std::vector<Table<2, unsigned int>> &       copy_indices,
     std::vector<Table<2, unsigned int>> &       copy_indices_global_mine,
@@ -284,11 +284,7 @@ void
 MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number>>::
   fill_and_communicate_copy_indices(const DoFHandler<dim, spacedim> &mg_dof)
 {
-  const parallel::TriangulationBase<dim, spacedim> *ptria =
-    dynamic_cast<const parallel::TriangulationBase<dim, spacedim> *>(
-      &mg_dof.get_triangulation());
-  const MPI_Comm mpi_communicator =
-    ptria != nullptr ? ptria->get_communicator() : MPI_COMM_SELF;
+  const MPI_Comm mpi_communicator = mg_dof.get_communicator();
 
   fill_internal(mg_dof,
                 mg_constrained_dofs,
